/*
 * Copyright (c) 2005 Michael Deryugin <dem@pikenet.ru>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef QSTRING_H
#define QSTRING_H

#include <sys/types.h>

#define MALLOC(ptr) palloc((ptr))
#define FREE(ptr) pfree((ptr))

enum qchars { Q, T, F, SP };

typedef struct {
	int length;
	char data[1];
} ipaggr;

struct qstring {
	int position;
	int qlength;
	int alength;
	ipaggr *qstring_data;
#define qdata		qstring_data->data
};

struct qstate {
	u_long addr;
	int mask;
	enum qchars qc;
	int state;
	struct qstring qstr;
#define STACK_SIZE 32
	int sp;
	int stack[STACK_SIZE];
};

/* Error */
#define ESYM (-1)

/* Actions of qwalk */
#define SEARCH_T	1
#define SEARCH_F	2
#define SEARCH_ADDR	3
#define SET_BLANK	4
#define AGGREGATE	5


/* qstring.c */
int	 qget(struct qstring *);
int	 qput(struct qstring *, enum qchars);
int	 qwalk(struct qstate *, int, int, u_long);
void	 qstate_init(ipaggr *, struct qstate *);
ipaggr *qgenerate(int, int, u_long, enum qchars);
ipaggr *qadd(ipaggr *, int, u_long);
ipaggr *qsub(ipaggr *, int, u_long);


/* log.c */
void             log_warn(const char *);
void             log_fatal(const char *);

#endif /* QSTRING_H */
